  .inesprg 8   ; 8x 16KB PRG code = 128KB
  .ineschr 0   ; 0x  8KB CHR data = CHR RAM
  .inesmap 1   ; mapper 1 = MMC1
  .inesmir %10   ; bottom bit for mirroring is completely ignored, mirroring must be set in your code
                 ; however the top bit is used to enable the WRAM battery

;;;;;;;;;;;;;;;

;; DECLARE SOME VARIABLES HERE
  .rsset $0000  ;;start variables at ram location 0
  
gamestate     .rs 1  ;current state
palettenum    .rs 1  ; which palette to use every frame

currentpage    .rs 1  ;which page to display
currentoption1 .rs 1  ;which page each option line links to
currentoption2 .rs 1
maxpage        .rs 1  ;highest page number
option1enabled .rs 1
option2enabled .rs 1


sourceLo      .rs 1  ; some pointers for loading text info
sourceHi      .rs 1
sourceBank    .rs 1

buttons       .rs 1  ;current controller buttons
newbuttons    .rs 1  ;which controller buttons have just been pressed

result        .rs 1  ;some temporary variables used for random code
temp          .rs 1

tempdec       .rs 5  ;used for hex->decimal conversions
temphex       .rs 2

framecounter  .rs 1  ;used for some delays like button repeating


  .rsset $0100
stack         .rs 256 ;;not actually needed, just a place holder

  .rsset $0200
sprites       .rs 256  ;sprite ram area



;;;;These variables have been moved to the WRAM area at $6000-7FFF
;; they are set up normally just like the console RAM, but cannot be used until the WRAM is enabled

  .rsset $6000
pagebank      .rs 256  ;data table for bank of each page text
pagehi        .rs 256  ;data table for high byte of address for start of each page text
pagelo        .rs 256
option1       .rs 256  ;data table for which page each option links to
option2       .rs 256

resetSignature  .rs 2    ;;signature in WRAM to detect how many times the console was reset
resetCount      .rs 1    ;;if the battery is enabled, this will count up ever time the ROM is loaded or reset
                         ;;if the battery is disabled, this will clear when the ROM is loaded but count up when it is reset



;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;
;; DECLARE SOME CONSTANTS HERE
STATETITLE     = $00  ; displaying title screen
STATEFADEOUT   = $01  ; fade to black
STATELOADPAGE  = $02  ; copy page to screen
STATEFADEIN    = $03  ; fade from black
STATESHOWPAGE  = $04  ; move paddles/ball, check for collisions


;;constants for page num sprite position
PAGEX   = $1C
PAGEY   = $07
;;;;;;;;;;;;;;;;;;


  .bank 0
  .org $8000
  .incbin "story.txt"

  .bank 1
  .org $A000

  .bank 2
  .org $8000

  .bank 3
  .org $A000

  .bank 4
  .org $8000

  .bank 5
  .org $A000

  .bank 6
  .org $8000

  .bank 7
  .org $A000

  .bank 8
  .org $8000

  .bank 9
  .org $A000

  .bank 10
  .org $8000

  .bank 11
  .org $A000

  .bank 12
  .org $8000

  .bank 13
  .org $A000

  .bank 14
  .org $C000   ;;8KB graphics in this fixed bank
Graphics:
  .incbin "graphics.nes"
  .incbin "graphics.nes"

  .bank 15    ;all code will go in the last 8KB, which is not a swappable bank
  .org $E000    



;;;;;;;;;;;;;;;;;;;;;
;;;;all game code from here down



;;;;first some subroutines


vblankwait:   ;;manually wait for a vblank to happen
  BIT $2002
  BPL vblankwait
  RTS


;;;;;;;;;;;;;;;;;

LoadSpritePalette:      ;;set correct address, then jump to the palette loading loop
  LDA $2002             ; read PPU status to reset the high/low latch
  LDA #$3F
  STA $2006             ; write the high byte of $3F10 address
  LDA #$10
  STA $2006             ; write the low byte of $3F10 address
  jmp LoadPalette


LoadBGPalette:          ;;set correct address, then jump to the palette loading loop
  LDA $2002             ; read PPU status to reset the high/low latch
  LDA #$3F
  STA $2006             ; write the high byte of $3F00 address
  LDA #$00
  STA $2006             ; write the low byte of $3F00 address



LoadPalette
  LDA palettenum        ;load which palette number to use
  CLC
  ASL A
  ASL A                 ;each palette is 16 bytes
  ASL A                 ;so shift 4 times to multiply by 16
  ASL A
  TAY                   ;transfer A to Y, because Y will use used as index below
   
  LDX #$00              ; byte count starts at 0
LoadPalettesLoop:
  LDA palettes, y       ; load data from address (palette + the value in y)
  STA $2007             ; write to PPU
  INY
  INX                   ; X = X + 1
  CPX #$10              ; Compare X to hex $10 = 16 bytes
  BNE LoadPalettesLoop  ; Branch to LoadPalettesLoop if compare was Not Equal to zero
                        ; if compare was equal to 16, all done
  RTS
  
  
  
  
;;;;;;;;;;;;;;;;;;;;
  
  
LoadCHRRAM:            ;;copies 8KB of graphics from PRG to CHR RAM
  lda $2002
  lda #$00
  sta $2006            ;set PPU to the CHR RAM area $0000-1FFF
  sta $2006
  ldy #$00
  ldx #$20             ;32 x 256 bytes = 8 KB
  lda #LOW(Graphics)
  sta sourceLo
  lda #HIGH(Graphics)  ;get the address of the graphics data ($C000)
  sta sourceHi         ;put into our source pointer
LoadCHRRamLoop:
  lda [sourceLo], y    ;copy from source pointer
  sta $2007            ;to PPU CHR RAM area
  iny
  bne LoadCHRRamLoop   ;;loop 256 times
  inc sourceHi         ;;then increment the high address byte
  dex                  ;;do that 32 times
  bne LoadCHRRamLoop   ;;32 x 256 = 8KB
LoadCHRRamDone:
  rts
  
  
;;;;;;;;;;;;;;;;;;;;;;;


ConfigWrite:     ; make sure this is in the last PRG bank so the RTS doesn't get swapped away
  LDA #$80
  STA $8000      ; reset the shift register
  
  LDA #%00001110 ; 8KB CHR, 16KB PRG, $8000-BFFF swappable, vertical mirroring
  STA $8000      ; first data bit
  LSR A          ; shift to next bit
  STA $8000      ; second data bit
  LSR A          ; etc
  STA $8000
  LSR A
  STA $8000
  LSR A
  STA $8000
  RTS


PRGBankWrite:     ; make sure this is in last bank so it doesnt get swapped away
  LDA sourceBank  ; load bank number into A
  
  AND #%01111111  ; clear the WRAM bit so it is always enabled
  
  STA $E000       ; first data bit
  LSR A           ; shift to next bit
  STA $E000
  LSR A
  STA $E000
  LSR A
  STA $E000
  LSR A
  STA $E000
  RTS



;bankvalues:  ;;This is the old bank switching code for UNROM
;  .db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
;BankSwitch:
;  lda sourceBank
;  tay
;  sta bankvalues, y  ;;make sure byte written = byte in ROM so there is no bus conflict
;  rts
  
  
;;;;;;;;;;;;;;;;;;;;;;
  
IncSource:  ;;increments the source address in $8000-BFFF range
            ;;checks for bank overflow, switches bank if needed
            
  lda sourceLo
  clc
  adc #$01
  sta sourceLo
  lda sourceHi
  adc #$00
  sta sourceHi
  cmp #$C0            ;;if high byte = C0, then address = $C000 and bank overflowed
  bne IncSourceDone
IncSourceOverflow:
  lda #$80            ;;go to $8000 in next bank
  sta sourceHi
  inc sourceBank
  jsr PRGBankWrite
IncSourceDone:
  rts


;;;;;;;;;;;;;;;;;;;;;;;;

  
;;data tables for decimal -> hex conversions
hundreds:
  .db 0, 100, 200
tens:
  .db 0, 10, 20, 30, 40, 50, 60, 70, 80, 90
ones:
  .db 0, 1, 2, 3, 4, 5, 6, 7, 8, 9

DecToHex:  ;;decimal to hex conversion
           ;;source address points to first character of 3 digit decimal number
           ;;"result" will contain the hex answer
  jsr PRGBankWrite    ;;make sure bank is correct using sourceBank

  ldy #$00
  lda [sourceLo], y
  sec
  sbc #$30           ;convert from ascii to 0-9
  tax
  lda hundreds, x    ;;get the hundreds digit
  sta result
  
  jsr IncSource     ;;go to next character
  
  ldy #$00
  lda [sourceLo], y
  sec
  sbc #$30           ;convert from ascii to 0-9
  tax
  lda tens, x    ;;get the tens digit
  clc
  adc result     ;;add to hundreds
  sta result

  jsr IncSource     ;;go to next character
  
  ldy #$00
  lda [sourceLo], y
  sec
  sbc #$30           ;convert from ascii to 0-9
  tax
  lda ones, x    ;;get the ones digit
  clc
  adc result     ;;add to hundreds+tens
  sta result
  rts
  
  
  
  
HexToDec: 
   lda #$00 
   sta tempdec+0 
   sta tempdec+1 
   sta tempdec+2 
   sta tempdec+3 
   sta tempdec+4 
   ldx #$10 
BitLoop: 
   asl temphex+0    ;;loop thro each bit, adding the equivalent decimal number
   rol temphex+1 
   ldy tempdec+0 
   lda BitTable, y 
   rol a
   sta tempdec+0 
   ldy tempdec+1 
   lda BitTable, y 
   rol a
   sta tempdec+1 
   ldy tempdec+2 
   lda BitTable, y 
   rol a
   sta tempdec+2 
   ldy tempdec+3 
   lda BitTable, y 
   rol a
   sta tempdec+3 
   rol tempdec+4 
   dex 
   bne BitLoop 
   
   lda tempdec    ;;convert to ascii by adding the ascii value of '0'
   clc
   adc #$30
   sta tempdec
   lda tempdec+1
   clc
   adc #$30
   sta tempdec+1
   lda tempdec+2
   clc
   adc #$30
   sta tempdec+2
   lda tempdec+3
   clc
   adc #$30
   sta tempdec+3
   lda tempdec+4
   clc
   adc #$30
   sta tempdec+4
   
   rts 

;;data table for hex -> dec conversions
BitTable: 
   .db $00, $01, $02, $03, $04, $80, $81, $82, $83, $84


;;;;;;;;;;;;;;;;;;;;;;;;

  
SavePageAddress:   ;;saves the address of page text to data table
  ldy currentpage
  lda sourceLo
  sta pagelo, y
  lda sourceHi
  sta pagehi, y
  lda sourceBank
  sta pagebank, y
  rts
  
SavePageOptions:    ;;saves the option numbers to data table
  ldy currentpage
  lda currentoption1
  sta option1, y
  lda currentoption2
  sta option2, y
  rts


;;;;;;;;;;;;;;;;;;;;

;;page format:
;  main text TAB 
;  option 1 text TAB
;  option 1 number (3 digits) TAB 
;  option 2 text TAB 
;  option 2 number (3 digits) TAB


CalculatePages:    ;;calculate all the page offsets and option numbers
                   ;;will read all the data, parsing it for the TAB delimiters
                   ;;keeping track of the address and saving it in the data tables in ram
                   ;;this could be done by a PC app and coded into the ROM instead
  lda #$00
  sta sourceBank
  sta sourceLo
  lda #$01
  sta currentpage   ;;start at page 1, page 0 is unused because excel has no row 0
  lda #$80
  sta sourceHi     ;;start at $8000 of bank 0
  jsr PRGBankWrite    ;;make sure bank is correct
    
CalculatePagesLoop:
  jsr SavePageAddress    ;;save the current address before doing more searching
  
CalculatePagesMainText:   ;;loop until end of text (TAB) is found
  jsr IncSource           ;go to next byte
  ldy #$00
  lda [sourceLo], y
  cmp #$09                ;;ascii character for TAB
  bne CalculatePagesMainText
  
CalculatePagesOption1Text:
  jsr IncSource           ;go to next byte (was stopped at TAB)
  ldy #$00
  lda [sourceLo], y
  cmp #$09                ;;ascii character for TAB
  bne CalculatePagesOption1Text
  
CalculatePagesOption1Number:
  jsr IncSource           ;go to next byte (was stopped at TAB)
  jsr DecToHex            ;do the decimal to hex conversion
  lda result
  sta currentoption1      ;save the result
  jsr IncSource           ;go to next byte (was stopped end of number)

    
CalculatePagesOption2Text:
  jsr IncSource           ;go to next byte (was stopped at TAB)
  ldy #$00
  lda [sourceLo], y
  cmp #$09                ;;ascii character for TAB
  bne CalculatePagesOption2Text
           
CalculatePagesOption2Number:
  jsr IncSource           ;go to next byte (was stopped at TAB)
  jsr DecToHex            ;do the decimal to hex conversion
  lda result
  sta currentoption2      ;save the result
  
  jsr SavePageOptions

  jsr IncSource           ;go to next byte (was stopped at end of number)

;;if next character = FF, reached end of tab file, no more pages
  ldy #$00
  lda [sourceLo], y
  cmp #$FF               ;;;;;;THIS LINE MAY HAVE TO BE FIXED DEPENDING ON YOUR ASSEMBLER
  beq CalculatePagesDone

;;if next+1 character = 0D, no more pages
  ldy #$01
  lda [sourceLo], y
  cmp #$0D
  beq CalculatePagesDone


;;check if this page was the maximum (256)
  lda currentpage
  cmp #$FF
  beq CalculatePagesDone
  
;;all done with this page, go to next page and search again
  inc currentpage
  jmp CalculatePagesLoop
  
CalculatePagesDone:
  lda currentpage    ;save the last page number
  sta maxpage
  rts

           
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;           
           
           
DrawPageNumber:
;;set sprite tile numbers for current page
  lda currentpage
  sta temphex
  lda #$00
  sta temphex+1
  jsr HexToDec
  lda tempdec    ;;ones digit
  sta $020D
  lda tempdec+1  ;;tens digit
  sta $0209
  lda tempdec+2  ;;hundreds digit
  sta $0205
  rts 
 
 
;;;;;;;;;;;;;;;;;;;;;;;;


ClearScreen:        ;;set all background to $00
  lda $2002
  lda #$20
  sta $2006
  lda #$00
  sta $2006
  ldx #$04
  ldy #$00
ClearScreenLoop:
  sta $2007
  dey
  bne ClearScreenLoop
  dex
  bne ClearScreenLoop
  rts


;;;;;;;;;;;;;;;;;;;;;;

           
DrawPage:
  jsr ClearScreen
  lda $2002
  lda #$20
  sta $2006
  lda #$20
  sta $2006
  
  lda #'P'
  sta $2007
  lda #'a'
  sta $2007
  lda #'g'
  sta $2007
  lda #'e'
  sta $2007                 ;;draw page number header
  
  lda $2002
  lda #$20
  sta $2006
  lda #$40
  sta $2006  
DrawPageText:
  ldy currentpage
  lda pagebank, y
  sta sourceBank
  lda pagelo, y
  sta sourceLo
  lda pagehi, y
  sta sourceHi
  jsr PRGBankWrite             ;;set character pointer to correct position for this page
  
  ldx #$00                   ;;used for horiz position in line

  ldy #$00
  lda [sourceLo], y          ;;if first char is CR, skip it
  cmp #$0D
  bne DrawPageTextLoop
  jsr IncSource
  
  
DrawPageTextLoop:
  ldy #$00
  lda [sourceLo], y
  cmp #$09
  beq DrawPageTextLoopDone   ;;TAB found, all done with text
  cmp #$0A
  beq DrawPageTextLoopNext   ;;NL new line (ignore)
  cmp #$0D
  beq DrawPageTextNextLine   ;;CR carriage return
  
  sta $2007                  ;;write char to screen
  
DrawPageTextLoopNext:
  jsr IncSource              ;;go to next character
  inx
  cpx #$20                   ;;increment horizontal position
  bne DrawPageTextLoop       ;;check if at next line
  ldx #$00
  jmp DrawPageTextLoop
  
DrawPageTextNextLine:
  lda #' '
  sta $2007                  ;;write spaces to screen
  inx
  cpx #$20                   ;;until line ends
  bne DrawPageTextNextLine
  ldx #$00                   ;;then start next line
  jsr IncSource              ;;go to next character
  jmp DrawPageTextLoop

DrawPageTextLoopDone:
  jsr IncSource              ;;last char was TAB
   
  ldy #$00
  lda [sourceLo], y
  cmp #$09
  beq DrawPageOption1Done    ;;skip over option 1 if empty
  
  lda $2002
  lda #$23
  sta $2006
  lda #$60
  sta $2006
  lda #'A'
  sta $2007
  lda #':'
  sta $2007
DrawPageOption1:
  ldy #$00
  lda [sourceLo], y
  cmp #$09
  beq DrawPageOption1Done   ;;TAB found, all done with text
  
  sta $2007                  ;;write char to screen
  jsr IncSource
  jmp DrawPageOption1
DrawPageOption1Done:
  jsr IncSource            ;;last char was tab
  
DrawPageSkipOptionNum:
  ldy #$00
  lda [sourceLo], y
  cmp #$09
  beq DrawPageSkipOptionNumDone    ;;skip over option 1 page number
  jsr IncSource
  jmp DrawPageSkipOptionNum
DrawPageSkipOptionNumDone:
  jsr IncSource
  
  ldy #$00
  lda [sourceLo], y
  cmp #$09
  beq DrawPageOption2Done    ;;skip over option 2 if empty

  lda $2002
  lda #$23
  sta $2006
  lda #$80
  sta $2006
  lda #'B'
  sta $2007
  lda #':'
  sta $2007
DrawPageOption2:
  ldy #$00
  lda [sourceLo], y
  cmp #$09
  beq DrawPageOption2Done   ;;TAB found, all done with text
  
  sta $2007                  ;;write char to screen
  jsr IncSource
  jmp DrawPageOption2
DrawPageOption2Done:  
  rts
            
            
;;;;;;;;;;;;;;;;;;;;;
            
            
DrawVersion:
  lda $2002
  lda #$20
  sta $2006
  lda #$40
  sta $2006

  ldy #$00
DrawVersionLoop:
  lda version, y
  sta $2007
  iny 
  cpy #versionend-version
  bne DrawVersionLoop
  rts
  
version:
  .db "CYOA v0.10   www.retrousb.com"
versionend:
            

;;;;;;;;;;;;;;;;;;;;;           
            
            
DrawResetCount:
  lda resetSignature           ;;read the signature bytes.  This checks if the WRAM was ever used before
  cmp #$AA                   ;; A sequence other than $AA $55 means this is the first time for WRAM.
  bne DrawResetCountClear          
  lda resetSignature+1         
  cmp #$55                 
  bne DrawResetCountClear
  jmp DrawResetCountNumber    ;;sequence was found, so WRAM was used before     


DrawResetCountClear:
  lda #$AA
  sta resetSignature          ;;sequence was not found, so write it and set the count to 0
  lda #$55
  sta resetSignature+1
  lda #'0'
  sta resetCount

DrawResetCountNumber:
  lda $2002
  lda #$20
  sta $2006
  lda #$5F
  sta $2006
  lda resetCount        ;;put the reset count at the end of the version line
  sta $2007
  
  clc
  adc #$01
  sta resetCount       ;;increment the counter, save into WRAM
  
  rts
           

;;;;;;;;;;;;;;;;;;;;;
           
           
           

RESET:
  SEI          ; disable IRQs
  CLD          ; disable decimal mode
  LDX #$40
  STX $4017    ; disable APU frame IRQ
  LDX #$FF
  TXS          ; Set up stack
  INX          ; now X = 0
  STX $2000    ; disable NMI
  STX $2001    ; disable rendering
  STX $4010    ; disable DMC IRQs


  jsr vblankwait    ; First wait for vblank to make sure PPU is ready



  jsr ConfigWrite   ; Set up the MMC1 banking and config

  lda #$00
  sta sourceBank
  jsr PRGBankWrite  ; do this switch to enable WRAM

  ;;8KB CHR RAM is used, so no banking or switching is needed
  ;;those registers are left alone



clrmem:
  LDA #$00
  STA $0000, x
  STA $0100, x
  STA $0300, x
  STA $0400, x
  STA $0500, x
  STA $0600, x
  STA $0700, x
  
  STA $6000, x
  STA $6100, x
  STA $6200, x
  STA $6300, x
  STA $6400, x  ;;clear out the WRAM we use too
  
  LDA #$FE
  STA $0200, x
  INX
  BNE clrmem
   
  jsr vblankwait      ; Second wait for vblank, PPU is ready after this


  


;;load all the graphics information
  lda #$00
  sta palettenum
  jsr LoadCHRRAM        
  jsr LoadBGPalette
  jsr LoadSpritePalette

  
;;calculate all the text offsets and page option numbers
  jsr CalculatePages
  
;;set up title page number display
  lda #PAGEY
  sta $0204
  lda #$00
  sta $0205
  lda #$01
  sta $0206
  lda #PAGEX+$08
  sta $0207
  
  lda #PAGEY
  sta $0208
  lda #$00
  sta $0209
  lda #$01
  sta $020A
  lda #PAGEX+$10
  sta $020B
  
  lda #PAGEY
  sta $020C
  lda #$00
  sta $020D
  lda #$01
  sta $020E
  lda #PAGEX+$18
  sta $020F

  lda #$01
  sta currentpage
  jsr DrawPageNumber
  jsr DrawPage
  jsr DrawVersion

  jsr DrawResetCount  ;;read from WRAM to see how many times the system was reset

;;:Set starting game state, change to intro when finished
  LDA #STATETITLE
  STA gamestate

  jsr vblankwait   ;;wait for vblank so screen isnt turned on while rendering is happening
  
  LDA #%10010000   ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1
  STA $2000

  LDA #%00011110   ; enable sprites, enable background, no clipping on left side
  STA $2001

  LDA #$00        ;;tell the ppu there is no background scrolling
  sta $2006
  sta $2006
  STA $2005
  STA $2005
  
  

Forever:
  JMP Forever     ;jump back to Forever, infinite loop, waiting for NMI
  
 

NMI:
  LDA #$00
  STA $2003       ; set the low byte (00) of the RAM address
  LDA #$02
  STA $4014       ; set the high byte (02) of the RAM address, start the transfer


  jsr LoadBGPalette
  jsr LoadSpritePalette

NMIDone:
  ;;This is the PPU clean up section, so rendering the next frame starts properly.
  LDA #%10010000   ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1
  STA $2000
  LDA #%00011110   ; enable sprites, enable background, no clipping on left side
  STA $2001
  LDA #$00        ;;tell the ppu there is no background scrolling
  STA $2005
  STA $2005
    
  ;;;all graphics updates done by here, run game engine

  lda framecounter
  cmp #$00
  beq FrameCounterDone
  dec framecounter     ;;decrement until 0, then stop
FrameCounterDone:  

  JSR ReadController1  ;;get the current button data for player 1


  
GameEngine:  
GameEngineTitle:
  lda gamestate
  cmp #STATETITLE
  bne GameEngineTitleDone
  jmp EngineTitle    ;;game is displaying title screen, checking controller for up/down/a/b
GameEngineTitleDone:

GameEngineFadeOut:
  lda gamestate
  cmp #STATEFADEOUT
  bne GameEngineFadeOutDone
  jmp EngineFadeOut    ;;game is fading to black
GameEngineFadeOutDone:

GameEngineFadeIn:
  lda gamestate
  cmp #STATEFADEIN
  bne GameEngineFadeInDone
  jmp EngineFadeIn    ;;game is from black to white
  GameEngineFadeInDone:

GameEngineLoadPage:
  lda gamestate
  cmp #STATELOADPAGE
  bne GameEngineLoadPageDone
  jmp EngineLoadPage    ;;game is loading current page
GameEngineLoadPageDone:

GameEngineShowPage:
  lda gamestate
  cmp #STATESHOWPAGE
  bne GameEngineShowPageDone
  jmp EngineShowPage    ;;game is showing a page, checking controller for A/B
GameEngineShowPageDone:

GameEngineDone:  
  RTI             ; return from interrupt
 
 
 
;;;;;;;;; 
 
EngineFadeOut:
  lda framecounter
  bne EngineFadeOutDone     ;;if framecounter still counting down, do nothing
  
  inc palettenum            ;;increment palette number, towards black
  lda #$04
  sta framecounter          ;;start the delay again    

  lda palettenum
  cmp #$05
  bne EngineFadeOutDone
  lda #$04                  ;;if palette = max palette 
  sta palettenum
  lda #STATELOADPAGE        ;;screen is black so load next page
  sta gamestate
EngineFadeOutDone:
  JMP GameEngineDone
 
;;;;;;;;;;;
  
EngineLoadPage:
  lda #$00
  sta $2000
  sta $2001             ;;turn screen off
  jsr DrawPage          ;;load new page, page number already set from options
  jsr DrawPageNumber    ;;load page number sprites
  jsr vblankwait
  LDA #%10010000        ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1
  STA $2000
  LDA #%00011110        ; enable sprites, enable background, no clipping on left side
  STA $2001
  lda #STATEFADEIN    ;;done loading, fade in
  sta gamestate
  JMP GameEngineDone

;;;;;;;;; 

EngineFadeIn:
  lda framecounter
  bne EngineFadeInDone     ;;if framecounter still counting down, do nothing
  
  dec palettenum            ;;decrement palette number, towards white text
  lda #$04
  sta framecounter          ;;start the delay again    

  lda palettenum
  cmp #$FF
  bne EngineFadeOutDone
  lda #$00                  ;;if palette = min palette 
  sta palettenum
  lda #STATESHOWPAGE        ;;screen is displayed so show page
  sta gamestate
EngineFadeInDone:
  JMP GameEngineDone

;;;;;;;;; 
 
 
EngineShowPage:
  lda newbuttons          ;;check for a new A press
  and #%10000000
  beq EngineShowPageNotA
  ldy currentpage
  lda option1, y
  beq EngineShowPageNotA  ;;if option1 page = 0, skip
  sta currentpage
  lda #STATEFADEOUT       ;;option 1 not zero, set up to load next page
  sta gamestate
  lda #$00
  sta framecounter        ;;start the fade immediately
  jmp EngineShowPageDone
EngineShowPageNotA:
  
  lda newbuttons          ;;check for a new B press
  and #%01000000
  beq EngineShowPageNotB
  ldy currentpage
  lda option2, y
  beq EngineShowPageNotB  ;;if option2 page = 0, skip
  sta currentpage
  lda #STATEFADEOUT       ;;option 2 not zero, set up to load next page
  sta gamestate
  lda #$00
  sta framecounter        ;;start the fade immediately
  jmp EngineShowPageDone
EngineShowPageNotB:

EngineShowPageDone:
  JMP GameEngineDone
 
;;;;;;;;;;;


;;;;;;;;
 
EngineTitle:
  
;;;check the up button to change page numbers
EngineTitleButtons:
  lda framecounter
  bne EngineTitleButtonsDone        ;;delay for button repeat

EngineTitleButtonsUp:
  lda buttons
  and #%00001000                    ;;check up button
  beq EngineTitleButtonsUpDone
  
  lda currentpage
  cmp maxpage
  bne EngineTitleButtonsUpDelay     ;;check if at end of book
  
  lda #$00
  sta currentpage                   ;;set to beginning of book
  
EngineTitleButtonsUpDelay:    
  inc currentpage                   ;;go to next page
  lda #$08
  sta framecounter                  ;;set delay for button repeat

EngineTitleButtonsUpDone:


;;check the down button to change page numbers
EngineTitleButtonsDown:
  lda buttons
  and #%00000100                    ;;check down button
  beq EngineTitleButtonsDownDone

  dec currentpage                   ;;go to next page
  lda currentpage
  cmp #$00
  bne EngineTitleButtonsDownDelay   ;;check if before beginning of book
  
  lda maxpage
  sta currentpage                   ;;set to end of book
  
EngineTitleButtonsDownDelay:    
  lda #$08
  sta framecounter                  ;;set delay for button repeat

EngineTitleButtonsDownDone:



EngineTitleButtonsStart:
  lda buttons
  and #%00010000                    ;;check start button
  beq EngineTitleButtonsStartDone
  lda #STATEFADEOUT                 ;;jump to selected page
  sta gamestate
  lda #$00
  sta framecounter                  ;;start the fade immediately
  jmp EngineShowPageDone
EngineTitleButtonsStartDone:


EngineTitleButtonsDone:
  jsr DrawPageNumber      ;;set page number sprites
  JMP EngineShowPage     ;;reuse the show page code to check controller A/B buttons



;;;;;;;;;;;;;;;;;;;;
     
;bit:   	7	6	5       4     3   2     1     0
;button:	A	B	select	start	up	down	left	right

ReadController1:
  LDA #$01
  STA $4016
  LDA #$00
  STA $4016
  LDX #$08
ReadController1Loop:   ;;read controller, save into temporary variable
  LDA $4016
  LSR A            ; bit0 -> Carry
  ROL temp        ; bit0 <- Carry
  DEX
  BNE ReadController1Loop

  lda temp                 ;;compare current buttons
  eor buttons              ;;to previous buttons
  and temp                 ;;only looking at buttons pressed now
  sta newbuttons           ;;to figure out which are new
  
  lda temp
  sta buttons              ;;save current buttons
  
  RTS
    
        
;;;;;;;;;;;;;;  
  
  
  
palettes:
  .db $1D,$30,$30,$30, $1D,$30,$30,$30, $1D,$30,$30,$30, $1D,$30,$30,$30   ;; palette 0
  .db $1D,$20,$1D,$1D, $1D,$20,$1D,$1D, $1D,$20,$1D,$1D, $1D,$20,$1D,$1D   ;; palette 1
  .db $1D,$10,$1D,$1D, $1D,$10,$1D,$1D, $1D,$10,$1D,$1D, $1D,$10,$1D,$1D   ;; palette 2
  .db $1D,$00,$1D,$1D, $1D,$00,$1D,$1D, $1D,$00,$1D,$1D, $1D,$00,$1D,$1D   ;; palette 3
  .db $1D,$1D,$1D,$1D, $1D,$1D,$1D,$1D, $1D,$1D,$1D,$1D, $1D,$1D,$1D,$1D   ;; palette 4






  .org $FFFA     ;first of the three vectors starts here
  .dw NMI        ;when an NMI happens (once per frame if enabled) the 
                   ;processor will jump to the label NMI:
  .dw RESET      ;when the processor first turns on or is reset, it will jump
                   ;to the label RESET:
  .dw 0          ;external interrupt IRQ is not used in this tutorial
  
  
;;;;;;;;;;;;;;  